--[[
    TotemManager
    Copyright (C) 2008 Gael Lalire
    Contact: gael.lalire@laposte.net

    TotemManager is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TotemManager is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with self.  If not, see <http://www.gnu.org/licenses/>.
--]]

-- begin import
local factoryLib = LibStub("FactoryLib-1.0");
local Factory = factoryLib.Factory;
local environmentLib = LibStub("EnvironmentLib-1.0");
local environment = environmentLib.environment;

local timerLib = LibStub("TimerLib-1.0");
local timerUpdaterLib = LibStub("TimerUpdaterLib-1.0");
local candyBarTimerRendererLib = LibStub("CandyBarTimerRendererLib-1.0");

local actionLib = LibStub("ActionLib-1.0");
local actionBarLib = LibStub("ActionBarLib-1.0");

local ActionBarRenderer = actionBarLib.ActionBarRenderer;

local managerFrameLib = LibStub("ManagerFrameLib-1.0");
-- end import


-- Ankh cooldown updater (only shaman)
local AnkhTimerUpdater, ankhTimerUpdater = factoryLib.Factory.CreateSingletonType(timerLib.TimerUpdater);

function AnkhTimerUpdater:UNIT_INVENTORY_CHANGED(timer)
  timer:SetCount(GetItemCount(self.ankh));  
end

TotemManager, totemManager = factoryLib.Factory.CreateSingletonType();

totemManagerData = {};

function TotemManager:BindButton(button, shortcut1, shortcut2, mouseClick)
  if button then
    ClearOverrideBindings(button);
    if shortcut1 then
      SetOverrideBindingClick(button, false, shortcut1, button:GetName(), mouseClick);
    end
    if shortcut2 then
      SetOverrideBindingClick(button, false, shortcut2, button:GetName(), mouseClick);
    end
  end
end

function TotemManager:AddTotemBind(totemName, totemSlot, bindingName)
  local shortcut1, shortcut2 = GetBindingKey(bindingName);
  self:BindButton(getglobal(totemName.."TotemSelected"), shortcut1, shortcut2, "C");
end


function TotemManager:UPDATE_BINDINGS()
  local shortcut1, shortcut2;
  
  self:AddTotemBind("air", 4, "TOTEM_MANAGER_AIR");
  self:AddTotemBind("water", 3, "TOTEM_MANAGER_WATER");
  self:AddTotemBind("fire", 1, "TOTEM_MANAGER_FIRE");
  self:AddTotemBind("earth", 2, "TOTEM_MANAGER_EARTH");
  
  shortcut1, shortcut2 = GetBindingKey("TOTEM_MANAGER_EARTH_SHIELD");
  self:BindButton(getglobal("earthShieldSelected"), shortcut1, shortcut2, "C");

  shortcut1, shortcut2 = GetBindingKey("TOTEM_MANAGER_WEAPON");
  self:BindButton(getglobal("weaponSelected"), shortcut1, shortcut2, "C");

  shortcut1, shortcut2 = GetBindingKey("TOTEM_MANAGER_SHIELD");
  self:BindButton(getglobal("shieldSelected"), shortcut1, shortcut2, "C");

end

function TotemManager:LoadDefaultData()
  self.tooltipShown = true;
  -- these data are in TotemManagerBackup
  totemManagerData = self.defaultTotemManagerData;
end

function TotemManager:Reset()
  self:LoadDefaultData();
  self.console:Print("Configuration reset, you need to restart your UI");
end

function TotemManager:SetTooltipVisibility(_, value)
  ActionBarRenderer.tooltipVisibility = tonumber(value);
  totemManagerData.tooltipVisibility = value;
end

function TotemManager:GetTooltipVisibility()
  return ActionBarRenderer.tooltipVisibility;
end

function TotemManager:IsLocked()
  return self.locked;
end

function TotemManager:ToggleLocked()
  self.locked = not self.locked;
  for _, frame in pairs(self.frames) do
    frame:ToggleLocked();
  end
end

function TotemManager:GetSkin()
  return totemManagerData.skin;
end

function TotemManager:SetSkin(_, value)
  totemManagerData.skin = tonumber(value);
end

function TotemManager:FirstActivation()
  self:LoadDefaultData();
  self:Activate();
  
  self.console:Print("Successful installation");
end

function TotemManager:CreateConsole()

  local options = {
    type = "group",
    handler = self;
    args = {
       skin = {
         type = "select",
         name = "skin",
         desc = "change skin (need to restart UI)",
         get = "GetSkin",
         set = "SetSkin",
         values = {"TM2", "Candy bar"},
         order = 1,
       },
       lock = {
         type = "toggle",
         name = "locked",
         desc = "lock or unlock all frames",
         set = "ToggleLocked",
         get = "IsLocked"
       },
       tooltip = {
         type = "select",
         name = "tooltip",
         desc = "tooltip visibility",
         get = "GetTooltipVisibility",
         set = "SetTooltipVisibility",
         values = {"Hide while in combat", "Show", "Hide"},
         order = 1,
       },
       --[[
       reset = {
         type = "execute",
         name = "reset",
         func = "Reset",
         desc = "reset configuration"
       }, ]]
    }    
  };
  
  for frameName, frame in pairs(self.frames) do
    options.args[frameName] = {type="group", name=frameName, handler=frame, args = {
      --[[
      visibility = {
        type = "toggle",
        name = "visible",
        desc = "show or hide this frame",
        set = "ToggleVisibility",
        get = "IsVisible"        
      }, ]]
      mode = {
        type = "select",
        name = "mode",
        desc = "direction of manager",
        get = "GetDirection",
        set = "SetDirection",
        values = {"Twisted Left mode", "Twisted Right mode", "Left mode", "Right mode"},
        order = 1,
      },
    }};
    
  end
  
  local console = {};
  self.console = console;
  setmetatable(console, {__index = LibStub("AceConsole-3.0"), __tostring = function ()
    return "TotemManager";
  end});
  
  LibStub("AceConfig-3.0"):RegisterOptionsTable("TotemManager", options);
  LibStub("AceConfigDialog-3.0"):AddToBlizOptions("TotemManager", "TotemManager");
  console:RegisterChatCommand("totemmanager", "ChatCommand");
  console:RegisterChatCommand("tm", "ChatCommand");

  function console:ChatCommand(input)
     LibStub("AceConfigCmd-3.0").HandleCommand(totemManager, "tm", "TotemManager", input);
  end  

end

function TotemManager:Activate()
  ActionBarRenderer.tooltipVisibility = totemManagerData.tooltipVisibility;
  
  ankhTimerUpdater.ankh = self.ANKH;
  


  self.locked = true;
  
  -- hide default blizzard timer
  TotemFrame:UnregisterEvent("PLAYER_TOTEM_UPDATE");
  TotemFrame:UnregisterEvent("PLAYER_ENTERING_WORLD");  
  TotemFrame:Hide();
  
  if totemManagerData.skin==2 then
    managerFrameLib.ManagerFrame.timerSkin = {visualRenderer = candyBarTimerRendererLib.candyBarTimerRendererFactory, verticalGap=20, horizontalGap=220};
  end
  
  self.left = 300;
  self.top = 200;
 -- totemManagerData.actionBarBackup = {};

  self.actionBarConfList = {};
  self.removeMacroTextMap = {};
  
  local updater = timerUpdaterLib.totemTimerUpdaterFactory:Create(4);

  self.actionBarConfList["airTotem"] = {
    {name=self.GROUNDING, effect=self.GROUNDING_EFFECT, duration = 45, updater=updater},
    {name=self.NATURE_RESISTANCE, effect=self.NATURE_RESISTANCE_EFFECT, updater=updater},
    {name=self.SENTRY, duration = 300, updater=updater},
    {name=self.WINDFURY, effect=self.WINDFURY_EFFECT, updater=updater},
    {name=self.WRATH_AIR, effect=self.WRATH_AIR_EFFECT, updater=updater},
  }
  self.removeMacroTextMap["airTotem"] = "/script DestroyTotem(4)";
  
  updater = timerUpdaterLib.totemTimerUpdaterFactory:Create(3);

  self.actionBarConfList["waterTotem"] = {
    {name=self.DISEASE_CLEANSING, tickFrequency = 5, updater=updater},
    {name=self.FIRE_RESISTANCE, effect=self.FIRE_RESISTANCE_EFFECT, updater=updater},
    {name=self.HEALING_STREAM, tickFrequency = 2, updater=updater},
    {name=self.ENAMORED_WATER_SPIRIT, totemName=self.ANCIENT_MANA_SPRING, action="item", duration = 24, effect=self.ANCIENT_MANA_SPRING_EFFECT, updater=updater},
    {name=self.MANA_SPRING, tickFrequency = 2, updater=updater},
    {name=self.MANA_TIDE, duration = 12, tickFrequency = 3, updater=updater},
    {name=self.POISON_CLEANSING, tickFrequency = 5, updater=updater},
  }
  self.removeMacroTextMap["waterTotem"] = "/script DestroyTotem(3)";

  updater = timerUpdaterLib.totemTimerUpdaterFactory:Create(1);

  self.actionBarConfList["fireTotem"] = {
    {name=self.FIRE_NOVA, duration = function ()
        local talentRank = environment:GetTalentRank(self.FIRE_TOTEM_TALENT, 1);
        return (5-talentRank);
      end, updater=updater
    },
    {name=self.FROST_RESISTANCE, effect=self.FROST_RESISTANCE_EFFECT, updater=updater},
    {name=self.FLAMETONGUE, effect=self.FLAMETONGUE_EFFECT, updater=updater},
    {name=self.MAGMA, duration = 20, tickFrequency = 2, updater=updater},
    {name=self.SEARING, duration = function (rank)
            return 5*(5+rank);
          end, updater=updater
    },
    {name=self.WRATH, effect=self.WRATH_EFFECT, updater=updater},
    {name=self.FIRE_ELEMENTAL, updater=updater},
  }
  self.removeMacroTextMap["fireTotem"] = "/script DestroyTotem(1)";

  updater = timerUpdaterLib.totemTimerUpdaterFactory:Create(2);

  self.actionBarConfList["earthTotem"] = {
    {name=self.EARTHBIND, duration = 45, tickFrequency = 3, tickStart = .2, updater=updater},
    {name=self.STONECLAW, duration = 15, tickFrequency = 2, life = function (rank)
            local level = UnitLevel("player");
            local base = 0;
            if rank==1 then base = 26;
            elseif rank==2 then base = 96;
            elseif rank==3 then base = 136;
            elseif rank==4 then base = 166;
            elseif rank==5 then base = 246;
            elseif rank==6 then base = 306;
            elseif rank==7 then base = 1114;
            end
            local talentRank = environment:GetTalentRank(self.EARTH_TOTEM_TALENT, 1);
            return (base+3*level)*(4+talentRank)/4;
          end, updater=updater
    },
    {name=self.STONESKIN, effect=self.STONESKIN_EFFECT, updater=updater},
    {name=self.STRENGTH_OF_EARTH, effect=self.STRENGTH_OF_EARTH_EFFECT, updater=updater},
    {name=self.TREMOR, tickFrequency = 3, updater=updater},
    {name=self.EARTH_ELEMENTAL, updater=updater},
  }
  self.removeMacroTextMap["earthTotem"] = "/script DestroyTotem(2)";

  self.actionBarConfList["weapon"] = {
    {name=self.WINDFURY_WEAPON, duration = 1800},
    {name=self.ROCKBITER_WEAPON, duration = 1800},
    {name=self.FLAMETONGUE_WEAPON, duration = 1800},
    {name=self.FROSTBRAND_WEAPON, duration = 1800},
    {name=self.EARTHLIVING_WEAPON, duration = 1800},
  }

  local removeMacroTextMap = "/script LibStub(\"EnvironmentLib-1.0\").environment:CancelPlayerBuffs(";
  self.actionBarConfList["shield"] = {
    {name=self.LIGHTNING_SHIELD, duration = 600},
    {name=self.WATER_SHIELD, duration = 600},
    {name=self.EARTH_SHIELD, target="player", duration = 600},
  }

  for _,v in pairs(self.actionBarConfList["shield"]) do
    v.updater = timerUpdaterLib.buffTimerUpdaterFactory:Create(v.name);
    removeMacroTextMap = removeMacroTextMap.."\""..(v.buffName or v.name).."\" ,";
  end
  self.removeMacroTextMap["shield"] = removeMacroTextMap.." nil)";
  
  updater = timerUpdaterLib.friendUniqueBuffTimerUpdaterFactory:Create(self.EARTH_SHIELD, actionLib.spellActionFactory:Create(self.EARTH_SHIELD, "player"));

  self.actionBarConfList["earthShield"] = {
    {name=self.EARTH_SHIELD, duration = 600, updater=updater},
    {name=self.EARTH_SHIELD, target="focus", duration = 600, updater=updater},
    {name=self.EARTH_SHIELD, target="targettarget", duration = 600, updater=updater},
  }

  --[[ 
  -----------------
    GUI CREATION PART
  -----------------
  --]]
  
  self.frames = {};
  
  local texture, selectedTexture = "Interface\\AddOns\\TotemManager\\textures\\SerenityBase", "Interface\\AddOns\\TotemManager\\textures\\SerenityBaseGreen";


  local messages = {
    started=self.TOTEM_STARTED,
    stopping=self.TOTEM_STOPPING,
    stopped=self.TOTEM_STOPPED,
    countAtZero=self.TOTEM_COUNT_AT_ZERO
  };
  
  self.frames["totemActionFrame"] = managerFrameLib.managerFrameFactory:Create(
    self.wowFrame["totemActionFrame"], 
    {"airTotem", "waterTotem", "fireTotem", "earthTotem"},
    self.actionBarConfList,
    "totem",
    self.removeMacroTextMap,
    totemManagerData["totemActionFrame"],
    messages,
    texture, selectedTexture
  );

  messages = {
    started=self.BUFF_STARTED,
    stopping=self.BUFF_STOPPING,
    stopped=self.BUFF_STOPPED,
  };  
    
  self.frames["buffActionFrame"] = managerFrameLib.managerFrameFactory:Create(
    self.wowFrame["buffActionFrame"],
    {"weapon", "shield", "earthShield"},
    self.actionBarConfList,
    "buff",
    self.removeMacroTextMap,
    totemManagerData["buffActionFrame"],
    messages,
    texture, selectedTexture
  );
  
  self:UPDATE_BINDINGS();
  
  self:CreateConsole();
end

function TotemManager:PrepareFrame()
  -- frame are created before PLAYER_ENTERING_WORLD, so wow restore their position.
  self.wowFrame = {
    totemActionFrame = CreateFrame("Frame", "totemActionFrame", parent),
    buffActionFrame = CreateFrame("Frame", "buffActionFrame", parent)    
  };
  
  self.wowFrame.totemActionFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 280, -180);
  self.wowFrame.buffActionFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 280, -380);
  for _, frame in pairs(self.wowFrame) do    
    frame:SetMovable(true);
  end
end

function TotemManager:PLAYER_ENTERING_WORLD()
  
  if totemManagerData.alreadyActivate then
    -- check older version
    if totemManagerData["skin"]==nil then
      totemManagerData.skin = 2; -- older keep candy (new get TM2)
      if totemManagerData.tooltipVisibility==nil then
        totemManagerData["totemActionFrame"] = {direction=1, visible=true};
        totemManagerData["buffActionFrame"] = {direction=1, visible=true};
        totemManagerData.tooltipVisibility = 1;
      end
    end
    
    -- activate
    self:Activate();
  else
    self:FirstActivation();
    totemManagerData.alreadyActivate = true;
  end
  environmentLib.environment:UnregisterEvent("PLAYER_ENTERING_WORLD", self);
end

function TotemManager:VARIABLES_LOADED()
  local _, unitName = UnitClass("player");
  if unitName=="SHAMAN" then
    self:PrepareFrame();
    environmentLib.environment:RegisterEvent("PLAYER_ENTERING_WORLD", self);
    environmentLib.environment:UnregisterEvent("VARIABLES_LOADED", self);
  end
end

-- run the mod (wait for VARIABLES_LOADED event)
-- reasonable scale values 0.8-1.1
totemManager.scale = 1;
environmentLib.environment:RegisterEvent("VARIABLES_LOADED", totemManager);
environmentLib.environment:RegisterEvent("UPDATE_BINDINGS", totemManager);
