--[[
    TotemManager
    Copyright (C) 2008 Gael Lalire
    Contact: gael.lalire@laposte.net

    TotemManager is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TotemManager is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TotemManager.  If not, see <http://www.gnu.org/licenses/>.
--]]

--[[
  The Action class.
--]]

-- Action extends EventHandler
Action, ActionFactory = Factory.CreateAbstractType(EventHandler)

-- type of action
function Action:GetType()
  return self.type;
end

-- shift-type[-button] will be set to value
function Action:GetShiftValue()
  return self.shiftValue;
end

-- type[-button] will be set to value
function Action:GetValue()
  return self.value;
end

function Action:GetTarget()
  return self.target;
end

function Action:GetTexture()
  return self.texture;
end

function Action:GetCooldown()
  error("Action:GetCooldown must be overrided");
end

function Action:ShowInTooltip(tooltip)
  error("Action:ShowInTooltip must be overrided");
end

function Action:CanBeCast()
  error("Action:CanBeCast must be overrided");
end

function Action:IsAvailable()
  self:CheckAvailability(); -- the check is not necessary if event are registred
  return self.available;
end

-- send an event if availability changed
function Action:CheckAvailability()
  error("Action:CheckAvailability must be overrided");
end

--[[
  The ActionFactory class.
--]]

ActionFactory = {};

-- type of action
function ActionFactory:Create(actionType, value, shiftValue, target, texture)
  local action = self:NewInstance();
  action.type = actionType;
  action.value = value;
  action.shiftValue = shiftValue;
  action.target = target;
  action.texture = texture;
  return action;
end

--[[
  The SpellAction class.
--]]

-- SpellAction extends Action
SpellAction, SpellActionFactory, spellActionFactory = Factory.CreateType(Action);

-- send an event if availability changed
function SpellAction:CheckAvailability()
  if self.available then
    self:SPELLS_FORGOTTEN();
  else
    self:SPELLS_LEARNT();
  end
end

function SpellAction:OnUsed(eventName)
  if eventName == "ACTION_AVAILABILITY_CHANGED" then
    if self.available then
      environment:RegisterEvent("SPELLS_FORGOTTEN", self);
    else
      environment:RegisterEvent("SPELLS_LEARNT", self);    
    end
  elseif eventName == "ACTION_PERFORMED" then
    environment:RegisterEvent("UNIT_SPELLCAST_SENT", self);      
  end
end

function SpellAction:OnUnused(eventName)
  if eventName == "ACTION_AVAILABILITY_CHANGED" then
    if self.available then
      environment:UnregisterEvent("SPELLS_FORGOTTEN", self);
    else
      environment:UnregisterEvent("SPELLS_LEARNT", self);    
    end
  elseif eventName == "ACTION_PERFORMED" then
    if self.waitForCastResult then
      environment:UnregisterEvent("UNIT_SPELLCAST_STOP", self);      
      environment:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED", self);
      self.waitForCastResult = false;
    end
    environment:UnregisterEvent("UNIT_SPELLCAST_SENT", self);      
  end
end

function SpellAction:UNIT_SPELLCAST_SENT(unit, spellName, rank, target)
  if unit ~= "player" or spellName ~= self.spellName then
    return;
  end
  -- check because mana spring can have or not a rank
  if rank=="" and not environment:IsUnrankedSpell(spellName) then
    return;
  end
  local validTarget = false;
  if environment:UnitFullName(self.target)==target then
    validTarget = true;
  end
  if self.target=="target" and (target=="" or UnitName("target")==nil) then
    validTarget = true;
  end
  if validTarget then
    self.targetName = target;
    environment:RegisterEvent("UNIT_SPELLCAST_STOP", self);
    environment:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", self);
    self.waitForCastResult = true;
  end  
end

function SpellAction:UNIT_SPELLCAST_STOP(unit)
  if unit=="player" then
    environment:UnregisterEvent("UNIT_SPELLCAST_STOP", self);      
    environment:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED", self);
    self.waitForCastResult = false;
  end
end

function SpellAction:UNIT_SPELLCAST_SUCCEEDED(unit, spellName, rank)
  if unit=="player" then
    environment:UnregisterEvent("UNIT_SPELLCAST_STOP", self);      
    environment:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED", self);
    self.waitForCastResult = false;
    if spellName==self.spellName then
      self:Fire("ACTION_PERFORMED", self, environment:RankOfString(rank), self.targetName);
    end
  end
end

function SpellAction:SPELLS_LEARNT()
  if environment:GetSpellMaxRank(self.spellName)~=0 then
    -- if we learn the spell
    environment:UnregisterEvent("SPELLS_LEARNT", self);
    environment:RegisterEvent("SPELLS_FORGOTTEN", self);
    if self.value == nil then
      -- load data
      local name, rank, icon = GetSpellInfo(self.spellName);
      self.available = true;
      self.type = "spell";
      self.value = name;
      if rank=="" then
        self.shiftValue = name;    
      else
        self.shiftValue = name..SpellAction.RANK_ONE;
      end
      self.texture = icon;
    end
    self:Fire("ACTION_AVAILABILITY_CHANGED", true);
  end
end

function SpellAction:SPELLS_FORGOTTEN()
  if environment:GetSpellMaxRank(self.spellName)==0 then
    -- if we forget the spell
    environment:UnregisterEvent("SPELLS_FORGOTTEN", self);
    environment:RegisterEvent("SPELLS_LEARNT", self);
    self:Fire("ACTION_AVAILABILITY_CHANGED", false);
  end
end

function SpellAction:GetCooldown()
  return GetSpellCooldown(environment:GetSpellId(self.value), BOOKTYPE_SPELL);
end

function SpellAction:ShowInTooltip(tooltip)
  tooltip:SetSpell(environment:GetSpellId(self.value), BOOKTYPE_SPELL);
end

function SpellAction:CanBeCast()
  return (IsUsableSpell(self.value));
end

--[[
  The ItemAction class.
--]]

-- ItemAction extends Action
ItemAction, ItemActionFactory, itemActionFactory = Factory.CreateType(Action);

-- send an event if availability changed
function ItemAction:CheckAvailability()
  self:UNIT_INVENTORY_CHANGED();
end

function ItemAction:UNIT_INVENTORY_CHANGED()
  if IsEquippedItem(self.itemName) then
    if not self.available then
      -- if first available (ie self.value == nil then fill other data)    
      if self.value == nil then
        local name, itemLink, _, _, _, _, _, _, _, itemTexture = GetItemInfo(self.itemName);
        if name==nil then
          return; -- we do not changed availability while we can't access to item info
        end
        self.itemLink = itemLink; -- keep item link info for tooltip
        self.type = "item";
        self.value = self.itemName;
        self.shiftValue = self.itemName;
        self.texture = itemTexture;
      end
      self.available = true;
      self:Fire("ACTION_AVAILABILITY_CHANGED", true);
    end
  else
    if self.available then
      self.available = false;
      self:Fire("ACTION_AVAILABILITY_CHANGED", false);
    end    
  end
end


function ItemAction:UNIT_SPELLCAST_SENT(unit, spellName, rank, target)
  if unit ~= "player" then
    return;
  end
  -- a item spell has no rank
  local itemSpellName, itemSpellRank = GetItemSpell(self.itemName);
  if rank~=itemSpellRank or spellName~=itemSpellName then
    return;
  end
  local validTarget = false;
  if UnitName(self.target)==target then
    validTarget = true;
  end
  if self.target=="target" and (target=="" or UnitName("target")==nil) then
    validTarget = true;
  end
  if validTarget then
    self.targetName = target;
    environment:RegisterEvent("UNIT_SPELLCAST_STOP", self);
    environment:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", self);
    self.waitForCastResult = true;
  end  
end

function ItemAction:UNIT_SPELLCAST_STOP(unit)
  if unit=="player" then
    environment:UnregisterEvent("UNIT_SPELLCAST_STOP", self);      
    environment:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED", self);
    self.waitForCastResult = false;
  end
end

function ItemAction:UNIT_SPELLCAST_SUCCEEDED(unit, spellName)
  if unit=="player" then
    environment:UnregisterEvent("UNIT_SPELLCAST_STOP", self);      
    environment:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED", self);
    self.waitForCastResult = false;
    if spellName==self.itemSpellName then
      self:Fire("ACTION_PERFORMED", self, self.targetName);
    end
  end
end

function ItemAction:OnUsed(eventName)
  if eventName == "ACTION_AVAILABILITY_CHANGED" then
    environment:RegisterEvent("UNIT_INVENTORY_CHANGED", self);
  elseif eventName == "ACTION_PERFORMED" then
    environment:RegisterEvent("UNIT_SPELLCAST_SENT", self);      
  end
end

function ItemAction:OnUnused(eventName)
  if eventName == "ACTION_AVAILABILITY_CHANGED" then
    environment:UnregisterEvent("UNIT_INVENTORY_CHANGED", self);
  elseif eventName == "ACTION_PERFORMED" then
    if self.waitForCastResult then
      environment:UnregisterEvent("UNIT_SPELLCAST_STOP", self);      
      environment:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED", self);
      self.waitForCastResult = false;
    end
    environment:UnregisterEvent("UNIT_SPELLCAST_SENT", self);      
  end
end

function ItemAction:GetCooldown()
  return GetItemCooldown(self.itemLink);
end

function ItemAction:ShowInTooltip(tooltip)
  tooltip:SetHyperlink(self.itemLink);
end

function ItemAction:CanBeCast()
  return (IsUsableItem(self.value))
end

--[[
  The ActionFactory class.
--]]
ActionFactory = {};

actionFactory = {};
setmetatable(actionFactory, {__index = ActionFactory});

-- spellName : string
function SpellActionFactory:Create(spellName, target)
  local spellAction = self:NewInstance();
  
  if target==nil then
    target = "target";
  end
  spellAction.target = target;

  spellAction.spellName = spellName;  
  spellAction.waitForCastResult = false;
  local name, rank, icon = GetSpellInfo(spellName);
  if name == nil then
    spellAction.available = false;
  else
    spellAction.available = true;
    spellAction.type = "spell";
    spellAction.value = name;
    if rank=="" then
      spellAction.shiftValue = name;    
    else
      spellAction.shiftValue = name..SpellAction.RANK_ONE;
    end
    spellAction.texture = icon;
  end
  return spellAction;
end

-- itemName : string
function ItemActionFactory:Create(itemName, target, equipNeeded)
  local itemAction = self:NewInstance();
  equipNeeded = true; -- do not manage item directly usable from bag

  if target==nil then
    target = "target";
  end
  
  itemAction.target = "target";

  itemAction.itemName = itemName;
  itemAction.waitForCastResult = false;
  itemAction.available = false;
  itemAction.equipNeeded = equipNeeded;
  local name, itemLink, _, _, _, _, _, _, _, itemTexture = GetItemInfo(itemName);
  
  if name == nil then
    itemAction.available = false;
  else
    if equipNeeded then
      itemAction.available = IsEquippedItem(itemName);
    end

    itemAction.itemLink = itemLink; -- keep item link info for tooltip
    itemAction.type = "item";
    itemAction.value = itemName;
    itemAction.shiftValue = itemName;
    itemAction.texture = itemTexture;
  end
  return itemAction;
end

