--[[
    TotemManager
    Copyright (C) 2008 Gael Lalire
    Contact: gael.lalire@laposte.net

    TotemManager is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TotemManager is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TotemManager.  If not, see <http://www.gnu.org/licenses/>.
--]]


--[[
  The TimerRenderer class.
--]]

TimerRenderer = Factory.CreateAbstractType();

-- this should be valid sentence :
-- - $name on $target is expiring
-- - $name sur $target va expirer
-- - so target should be the name of person concerned by timer (and not "player" which belongs to english langage)
-- if $target is "" then this should be valid sentence :
-- - $name is expiring
function TimerRenderer:StartTimer(name, duration, texture, target, count, tickFrequency, tickStart)
  error("TimerRenderer:StartTimer must be overrided");
end

function TimerRenderer:StopTimer()
  error("TimerRenderer:StopTimer must be overrided");
end

function TimerRenderer:SetEnabled(enabled)
  error("TimerRenderer:SetEnabled must be overrided");
end

function TimerRenderer:SetCount(count)
end

--[[
  The Timer class.
--]]

Timer, TimerFactory = Factory.CreateAbstractType();

function Timer:StartTimer(name, duration, texture, target, count, tickFrequency, tickStart, updater)
  if self.updater ~= nil then
    self.updater:Disable(self);
  end
  duration = duration or 0;
  -- if duration is 0 (or less) then we do not need to register updater and show timer renderers
  if duration > 0 then
    if updater~=nil then
      updater:Enable(self);
    end
    self.updater = updater;
    -- set default values
    name = name or "";
    target = target or "";
    count = count or -1;
    tickFrequency = tickFrequency or -1;
    tickStart = tickStart or -1;
    for i=1, #self.timerRendererList do
      self.timerRendererList[i]:StartTimer(name, duration, texture, target, count, tickFrequency, tickStart);
    end
  end
end

-- stop timer is a brutal event (buff dispell, totem destroy)
-- it is always call once by one of updater handler list (because we unregister after)
function Timer:StopTimer()
  for i=1, #self.timerRendererList do
    self.timerRendererList[i]:StopTimer();
  end
  if self.updater~=nil then
    self.updater:Disable(self);
    self.updater = nil;
  end
end

-- count is an integer between 0 and 99 (or -1)
-- can mean total person buffed, charge remaining ...
function Timer:SetCount(number)
  for i=1, #self.timerRendererList do
    self.timerRendererList[i]:SetCount(number);
  end
end

--[[
  The TimerFactory class.
--]]

function TimerFactory:Create(timerRendererList)
  local timer = self:NewInstance();
  timer.timerRendererList = timerRendererList;
  return timer;
end

--[[
  The ActionPerformedTimer class.
--]]

-- ActionPerformedTimer extends Timer
ActionPerformedTimer, ActionPerformedTimerFactory, actionPerformedTimerFactory = Factory.CreateType(Timer);

-- action performed do not send anything by default (maybe the action itself)
function ActionPerformedTimer:ACTION_PERFORMED(actionProperty, actionFunc, ...)
  local timerProperties = actionProperty;
  if actionFunc~=nil then
    timerProperties = actionFunc(actionProperty, ...);
  end
  self:StartTimer(timerProperties.name, timerProperties.duration, timerProperties.texture, timerProperties.target, timerProperties.count,
    timerProperties.tickFrequency, timerProperties.tickStart, timerProperties.updater);
end


--[[
  The ActionPerformedTimerFactory class.
--]]

function ActionPerformedTimerFactory:Create(timerRendererList, actionList, actionPropertyMap, actionFuncMap)
  local actionPerformedTimer = self:NewInstance(timerRendererList);
  
  if actionPropertyMap == nil then
    error("TimerFactory:CreateActionPerformedTimer actionPropertyMap cannot be nil");
  end
  if actionFuncMap == nil then
    actionFuncMap = {};
  end
  
  for i=1, #actionList do
    local action = actionList[i];
    action:RegisterEvent("ACTION_PERFORMED", actionPerformedTimer, actionPropertyMap[action], actionFuncMap[action]);
  end
  
  return actionPerformedTimer;
end

--[[
  The ActionCooldownTimer class.
--]]

-- ActionCooldownTimer extends Timer
ActionCooldownTimer, ActionCooldownTimerFactory, ActionCooldownTimerFactory = Factory.CreateType(Timer);

function ActionCooldownTimerFactory:Create(timerRendererList, action)
  local actionCooldownTimer = self:NewInstance(timerRendererList);
  actionCooldownTimer.action = action;
  return actionCooldownTimer;
end

