--[[
    ActionBarLib
    Copyright (C) 2008 Gael Lalire
    Contact: gael.lalire@laposte.net

    ActionBarLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ActionBarLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ActionBarLib.  If not, see <http://www.gnu.org/licenses/>.
--]]

--[[
  The ActionBarRenderer class.
--]]

local actionBarLib = LibStub:NewLibrary("ActionBarLib-1.0", 1);
if not actionBarLib then
  return;
end

-- begin import
local factoryLib = LibStub("FactoryLib-1.0");
local Factory = factoryLib.Factory;
-- end import

local ActionBarRenderer = Factory.CreateAbstractType();
actionBarLib.ActionBarRenderer = ActionBarRenderer;

ActionBarRenderer.TOOLTIP_HIDE_IN_COMBAT = 1;
ActionBarRenderer.TOOLTIP_SHOW = 2;
ActionBarRenderer.TOOLTIP_HIDE = 3;

ActionBarRenderer.tooltipVisibility = 1;
ActionBarRenderer.tooltipHelpAdded = true;

function ActionBarRenderer:CreateSecureUnitButton(parent, position)
  error("ActionBarRenderer:CreateSecureUnitButton must be overrided");
end

function ActionBarRenderer:ActionChanged(secureUnitButton, action)
  error("ActionBarRenderer:ActionChanged must be overrided");
end

function ActionBarRenderer:IsTooltipAsked()
  if self.tooltipVisibility == 2 then
    return true;
  elseif self.tooltipVisibility == 3 then
    return false;
  else
    return not InCombatLockdown();
  end
end

function ActionBarRenderer:AddSelectedTooltipHelp(tooltip)
  tooltip:AddLine("Action bar selected button (bindable)", 0, 1, 0);
  tooltip:AddLine("Alt-leftClick : Add a favorite", 1, 1, 1);
  tooltip:AddLine("Shift-alt-leftClick : Remove a favorite", 1, 1, 1);
  tooltip:AddLine("RightClick : Open the bar", 1, 1, 1);
  tooltip:AddLine("LeftClick : Perform action", 1, 1, 1);
  tooltip:AddLine("Shift-leftClick : Perform action first rank", 1, 1, 1);
  tooltip:AddLine("Ctrl-leftClick : May cancel action result", 1, 1, 1);
end

function ActionBarRenderer:AddFavoriteTooltipHelp(tooltip)
  tooltip:AddLine("Action bar favorite button", 0, 1, 0);
  tooltip:AddLine("Rightclick : Exchange action with selected action", 1, 1, 1);
  tooltip:AddLine("Leftclick : Perform action", 1, 1, 1);
  tooltip:AddLine("Shift-leftclick : Perform action first rank", 1, 1, 1);
end

function ActionBarRenderer:AddOpenTooltipHelp(tooltip)
  tooltip:AddLine("Action bar button", 0, 1, 0);
  tooltip:AddLine("Ctrl-leftclick : Exchange action with next", 1, 1, 1);
  tooltip:AddLine("Shift-leftclick : Exchange action with previous", 1, 1, 1);
  tooltip:AddLine("Rightclick : Close the bar and select action", 1, 1, 1);
  tooltip:AddLine("Leftclick : Close the bar, select action and perform action", 1, 1, 1);
  tooltip:AddLine("Shift-leftclick : Close the bar, select action and perform action first rank", 1, 1, 1);
end


--[[
  The ActionBarState class.
--]]

local ActionBarState, ActionBarStateFactory, actionBarStateFactory = Factory.CreateType();

function ActionBarState:Open()
  if self.selectedVisibility == 0 then
    error("ActionBarState:Open already open");
  end
  self.selectedVisibility = 0;
  self.selectedAction = 0;
  local actionListSize = #self.visibilityList;
  for i=1, actionListSize do
    if self.visibilityList[i] == 4 then
      break;
    elseif self.visibilityList[i] == 2 then
      self.visibilityList[i] = 0;
    elseif self.visibilityList[i] == 3 then
      self.visibilityList[i] = 1;
    end
  end
  self.originalSelectedAction = 0;
end

function ActionBarState:Close(actionId)
  if self.selectedVisibility == 1 then
    error("ActionBarState:Close already closed");
  end
  self.selectedVisibility = 1;
  self.selectedAction = actionId;
  local actionListSize = #self.visibilityList;
  for i=1, actionListSize do
    if self.visibilityList[i] == 4 then
      break;
    elseif self.visibilityList[i] == 0 then
      self.visibilityList[i] = 2;    
    elseif self.visibilityList[i] == 1 then
      self.visibilityList[i] = 3;
    end
  end
  self.originalSelectedAction = actionId;
end

-- do not call directly
function ActionBarState:InsertAction(actionId, position, stucked)
  local actionListSize = #self.actionList;
  local positionOldVisibility = self.visibilityList[position];

  if positionOldVisibility==4 then
    -- insert at the end (imply no need to push button at the end)
    self.actionList[position] = actionId;
    if stucked then
      -- if previous is stuck
      if position == 1 or self.visibilityList[position - 1]%2 == 1 then
        if self.selectedVisibility==0 then
          self.visibilityList[position] = 1;
        else
          self.visibilityList[position] = 3;        
        end
      else
        -- we can't be stuck so we becomes like previous
        self.visibilityList[position] = self.visibilityList[position - 1];      
      end
    else
      -- no issue we set our value
      if self.selectedVisibility==0 then
        self.visibilityList[position] = 0;
      else
        self.visibilityList[position] = 2;        
      end
    end
    return;
  end  
  
  for i=1, actionListSize - position do
    self.actionList[actionListSize-i+1] = self.actionList[actionListSize-i];
  end
  self.actionList[position] = actionId;
  local stateReplaced = false;
  
  local positionNewVisibility = positionOldVisibility;

  if stucked then    
    if position == 1 or self.visibilityList[position - 1]%2 == 1 then
      -- if the previous is stucked then we can set our value
      if positionOldVisibility == 0 then
        self.visibilityList[position] = 1;
        positionNewVisibility = 1;
      elseif positionOldVisibility == 2 then
        self.visibilityList[position] = 3; 
        positionNewVisibility = 3;     
      end
    end
  end  
  
  local j = position + 1;
  -- first different
  while self.visibilityList[j] == positionNewVisibility do
    j = j + 1;
  end
  local secondPositionVisibility = self.visibilityList[j];

  -- if we get a non empty button
  if secondPositionVisibility~=4 then
    -- look for '4'
    local k = j + 1;
    while self.visibilityList[k] == secondPositionVisibility do
      k = k + 1;
    end
    self.visibilityList[k] = secondPositionVisibility;
    -- if we did not change our state
    if positionNewVisibility == positionOldVisibility then
      self.visibilityList[j] = positionOldVisibility;
    end
  else
    -- we find 4
    self.visibilityList[j] = positionOldVisibility;
  end
end

function ActionBarState:DeleteAction(position)
  local actionListSize = #self.actionList;
  for i=position, actionListSize-1 do
    self.actionList[i] = self.actionList[i+1];
    self.visibilityList[i] = self.visibilityList[i+1];
  end
  self.actionList[actionListSize] = 0;
  self.visibilityList[actionListSize] = 4;  
end

function ActionBarState:EnableAction(actionId)
  local unavailable = self.unavailable[actionId];
  local pos = unavailable[1];
  local i = pos;
  for _, v in pairs(self.unavailable) do
    if v[1] < unavailable[1] then
      i = i - 1;
    end
  end
  self:InsertAction(actionId, i, unavailable[2]);
  self.unavailable[actionId] = nil;
  if self.originalSelectedAction == actionId then
    self:Close(actionId);
  end
end

function ActionBarState:IsActionEnabled(actionId)
  return (self.unavailable[actionId] == nil);
end

function ActionBarState:SetActionEnabled(actionId, enabled)
  if enabled then
    self:EnableAction(actionId);
  else
    self:DisableAction(actionId);
  end
end

function ActionBarState:DisableAction(actionId)
  local i = 1;
  local actionListSize = #self.actionList;
  while i<=actionListSize and self.actionList[i] ~= actionId do
    i = i + 1;
  end
  if i>actionListSize then
    error("ActionBarState:DisableAction already disabled or inexistant");  
  end
  local defined = {};
  for _, v in pairs(self.unavailable) do
    defined[v[1]] = true;
  end

  local pos = 0;
  local j = 0; -- j = # button found
  while j<i do
    pos = pos + 1;
    if not defined[pos] then
      j = j + 1; -- there is a real button
    end
  end
  
  local stucked = (self.visibilityList[i]==1 or self.visibilityList[i]==3);
  
  self.unavailable[actionId] = {pos, stucked};
  self:DeleteAction(i);
  
  if self.selectedAction == actionId then
    self:Open();
    self.originalSelectedAction = actionId;
  end
end

function ActionBarStateFactory:Create()
  local actionBarState = self:NewInstance();
  actionBarState.selectedVisibility = 0;
  actionBarState.selectedAction = 0;
  actionBarState.originalSelectedAction = 0;
  actionBarState.visibilityList = {};
  actionBarState.actionList = {};
  actionBarState.unavailable = {};

  for i=1,actionListSize do
    actionBarState.visibilityList[i] = 0;
    actionBarState.actionList[i] = i;
  end  
end

--[[
  The ActionBar class.
--]]


local ActionBar, ActionBarFactory, actionBarFactory = Factory.CreateType();
actionBarLib.ActionBar = ActionBar;
actionBarLib.ActionBarFactory = ActionBarFactory;
actionBarLib.actionBarFactory = actionBarFactory;

function ActionBar:SpreadState()
  self.selectedSecureStateHeader:SetAttribute("state", self.state.selectedVisibility);
  self.selectedCasterSecureStateHeader:SetAttribute("state", self.state.selectedAction);
  for i=1, #self.state.visibilityList do
    self.secureStateHeaderList[i]:SetAttribute("state", self.state.visibilityList[i]);
    self.casterSecureStateHeaderList[i]:SetAttribute("state", self.state.actionList[i]);
  end
  --[[ spread also set state
  for setName, actionBarState in pairs(self.set) do
    self.:SetAttribute("*newstate-B"..setName, self.state.selectedVisibility);
    self.:SetAttribute("*newstate-B"..setName, self.state.selectedAction);

    for i=1, #self.state.visibilityList do
      self.[i]:SetAttribute("*newstate-B"..setName, self.state.visibilityList[i]);
      self.[i]:SetAttribute("*newstate-B"..setName, self.state.actionList[i]);
    end
  end
  ]]
end

-- disable an action (if selected and close we open)
function ActionBar:SetSelectedCtrlClickMacroText(macroText)
  self.selectedCasterSecureUnitButtonBis:SetAttribute("*macrotext*", macroText or "");
end


-- disable an action (if selected and close we open)
function ActionBar:DisableAction(actionPosition)
  -- check in combat
  local actionListSize = #self.actionList;
  if actionPosition<1 or actionPosition>actionListSize then
    error("intern error : actionPosition out of range");
    return;
  end
    
  self.state:DisableAction(actionPosition);
  self:SpreadState();  
end

-- simple add an action at the end (should evoluate to keep more information)
function ActionBar:EnableAction(actionPosition)
  -- check in combat
  local actionListSize = #self.actionList;
  if actionPosition<1 or actionPosition>actionListSize then
    error("intern error : actionPosition out of range");
    return;
  end
  
  self.state:EnableAction(actionPosition);
  self:SpreadState();
end



-- on click on this button will cast selected element
function ActionBar:GetSelectedButton()

end

-- returns {selectedAction=7, selectedVisibility=0, actionList={2, 1, 6, 7, 3, 0, 0}, visibilityList={2, 2, 2, 2, 2, 4, 4}}
-- this state can be given in constructor (actionBarState)
function ActionBar:GetBackup()
  return self.state;
end


function ActionBar:LoadActionData(i)
  local action = self.actionList[i];
  local actionType = action:GetType();
  local actionTarget = action:GetTarget();
  if actionTarget=="target" then
    actionTarget = ""; -- spell without target
  end
  self.selectedCasterSecureUnitButtonBis:SetAttribute("*newstate-W"..i,"*:"..i);
  -- receive C (Cast)
  self.selectedCasterSecureUnitButtonBis:SetAttribute("*type-C"..i, actionType);
  self.selectedCasterSecureUnitButtonBis:SetAttribute(actionType.."-C"..i, action:GetValue());
  self.selectedCasterSecureUnitButtonBis:SetAttribute("*unit-C"..i, actionTarget);
  self.selectedCasterSecureUnitButtonBis:SetAttribute("shift-"..actionType.."-C"..i, action:GetShiftValue());
  -- receive W (Write+Cast)
  self.selectedCasterSecureUnitButtonBis:SetAttribute("*type-W"..i, actionType);
  self.selectedCasterSecureUnitButtonBis:SetAttribute(actionType.."-W"..i, action:GetValue());
  self.selectedCasterSecureUnitButtonBis:SetAttribute("*unit-W"..i, actionTarget);
  self.selectedCasterSecureUnitButtonBis:SetAttribute("shift-"..actionType.."-W"..i, action:GetShiftValue()); 


  -- ctrl click support
  self.selectedCasterSecureUnitButtonBis:SetAttribute("ctrl-type-C"..i, "macro");

  self.dataLoaded[i] = true;
end

function ActionBar:ACTION_AVAILABILITY_CHANGED(actionPosition, available)
  if available then
    if not self.dataLoaded[actionPosition] then
      self:LoadActionData(actionPosition);
    end
    self:EnableAction(actionPosition);
  else
    self:DisableAction(actionPosition);
  end
end

local function debugSecureUnitButton(msg, secureUnitButton)

  secureUnitButton:SetScript("PreClick", function (b, c)
    DEFAULT_CHAT_FRAME:AddMessage("button "..msg.." receive '"..c.."'");
  end)
end

--[[
  The ActionBarFactory class.
--]]

function ActionBarFactory:Create(casterButtonName, actionList, actionBarRenderer, actionBarState)
  local actionBar = self:NewInstance();
  
  actionBar.actionList = actionList;
  
  local actionListSize = #actionList;
  
  if actionListSize == 0 then
    return nil;
  end


  -- this algo keep only available action but not all available action ()  
  if actionBarState==nil then
    actionBarState = actionBarStateFactory:Create();
  else
    actionBarStateFactory:Cast(actionBarState);
  end
  actionBar.state = actionBarState;
  
  for i=1, actionListSize do
    action = actionList[i];
    local actionAvailable = action:IsAvailable();
    local actionEnabled = actionBarState:IsActionEnabled(i);
    --     if actionEnabled ~= actionAvailable then BUG ??
    if actionEnabled and not actionAvailable then
      actionBarState:SetActionEnabled(i, actionAvailable);
    end
    if not actionEnabled and actionAvailable then
       actionBarState:SetActionEnabled(i, actionAvailable);
    end
  end
  
  -- use to send current action to selected button
  local hcString = "";
  local hString = "";
  local cString = "";
  local mString = "";
  local xString = "";
  for i=1,actionListSize do
    hcString = hcString..i..":HC"..i..";";
    hString = hString..i..":H"..i..";";
    cString = cString..i..":C"..i..";";
    mString = mString..i..":M"..i..";";
    xString = xString..i..":X"..i..";";
  end
  
  -- keep actionListSize state (one by action)
  local selectedCasterSecureStateHeader = CreateFrame("Frame", nil, UIParent, "SecureStateHeaderTemplate");
  selectedCasterSecureStateHeader.actionBar = actionBar;

  function selectedCasterSecureStateHeader:StateChanged(e)
    e = tonumber(e);
    actionBarRenderer:ActionChanged(self.actionBar.selectedSecureUnitButton, self.actionBar.actionList[e]);
    self.actionBar.state.selectedAction = e;
    if e~=0 then
      self.actionBar.state.originalSelectedAction = e;
    end
  end
  local selectedCasterSecureUnitButton = CreateFrame("Button", nil, parent, "SecureUnitButtonTemplate");
  selectedCasterSecureStateHeader:SetAttribute("addchild", selectedCasterSecureUnitButton);
  selectedCasterSecureUnitButton:SetAttribute("*type*", "click");
  for i=1,actionListSize do
    local m2String = "";
    for j=1,actionListSize do
      m2String = m2String..j..":"..j.."M"..i..";";
      selectedCasterSecureUnitButton:SetAttribute("*newstate-"..j.."M"..i,"*:"..i);
    end
    selectedCasterSecureUnitButton:SetAttribute("*newstate-H"..i,"*:"..i);
    selectedCasterSecureUnitButton:SetAttribute("*statebutton-M"..i, m2String);
  end
  --  debugSecureUnitButton("S?", selectedCasterSecureUnitButton);
  
  actionBar.dataLoaded = {};
  
  -- the only one button wich cast actions
  local selectedCasterSecureUnitButtonBis = CreateFrame("Button", casterButtonName, parent, "SecureUnitButtonTemplate");
  selectedCasterSecureStateHeader:SetAttribute("addchild", selectedCasterSecureUnitButtonBis);
  selectedCasterSecureUnitButtonBis:SetAttribute("*statebutton-C", cString);
  selectedCasterSecureUnitButtonBis:SetAttribute("*newstate-S","*:0");
  actionBar.selectedCasterSecureUnitButtonBis = selectedCasterSecureUnitButtonBis;
  for i=1,actionListSize do
    local action = actionList[i];
    if action:IsAvailable() then
      actionBar:LoadActionData(i);
    else
      actionBar.dataLoaded[i] = false;
    end
  end
  -- debugSecureUnitButton("SB?", selectedCasterSecureUnitButtonBis);

  selectedCasterSecureUnitButtonBis:SetAttribute("*macrotext*", "");


  -- keep 2 state (hidden => 0, visible => 1)
  local selectedSecureStateHeader = CreateFrame("Frame", nil, UIParent, "SecureStateHeaderTemplate");
  selectedSecureStateHeader:SetAllPoints();

  selectedSecureStateHeader.actionBar = actionBar;
  function selectedSecureStateHeader:StateChanged(e)
    e = tonumber(e);
    self.actionBar.state.selectedVisibility = e;
  end  
  
  local selectedSecureUnitButton = actionBarRenderer:CreateSecureUnitButton(selectedSecureStateHeader, 0);
  actionBar.selectedSecureUnitButton = selectedSecureUnitButton;
  selectedCasterSecureStateHeader:SetAttribute("state", tostring(actionBarState.selectedAction));
  selectedSecureStateHeader:SetAttribute("state", tostring(actionBarState.selectedVisibility));
  selectedSecureUnitButton:RegisterForClicks("LeftButtonUp", "RightButtonUp");
  selectedSecureUnitButton:SetAttribute("*type*", "click");
  selectedSecureUnitButton:SetAttribute("hidestates", "0");
  selectedSecureStateHeader:SetAttribute("addchild", selectedSecureUnitButton);
  for i=1,actionListSize do
    selectedSecureUnitButton:SetAttribute("*newstate-HC"..i,"*:1");
  end
  selectedSecureUnitButton:SetAttribute("*newstate-H","*:1");
  selectedSecureUnitButton:SetAttribute("*newstate-S","*:0");
  selectedSecureUnitButton:SetAttribute("*statebutton2", "S");
  selectedSecureUnitButton:SetAttribute("*statebutton1", "C");
  selectedSecureUnitButton:SetAttribute("alt-statebutton1", "A");
  selectedSecureUnitButton:SetAttribute("alt-shift-statebutton1", "R");
  selectedSecureUnitButton:SetAttribute("*clickbutton-C", selectedCasterSecureUnitButtonBis);



  for i=1,actionListSize do
    selectedSecureUnitButton:SetAttribute("*statebutton-H"..i, "H");
  end

  --  debugSecureUnitButton("T?", selectedSecureUnitButton);

  selectedCasterSecureUnitButton:SetAttribute("*clickbutton*", selectedSecureUnitButton);

  -- tmp variable to complete link between buttons
  local casterSecureUnitButtonList = {}; -- can access or modify action
  local casterSecureUnitButtonBisList = {}; -- if we need a second action with same header
  local secureUnitButtonList = {}; -- visible buttons (created by renderer)
  local secureUnitButtonBisList = {}; -- if we need a non user action on button state
  local casterSecureStateHeaderList = {};
  local secureStateHeaderList = {};
  
  for i=1,actionListSize do
    -- keep actionListSize state (one by action)
    local casterSecureStateHeader = CreateFrame("Frame", nil, UIParent, "SecureStateHeaderTemplate");
    casterSecureStateHeaderList[i] = casterSecureStateHeader;
    
    local casterSecureUnitButton = CreateFrame("Button", nil, parent, "SecureUnitButtonTemplate");
    casterSecureUnitButtonList[i] = casterSecureUnitButton;
    casterSecureStateHeader:SetAttribute("addchild", casterSecureUnitButton);
    
    casterSecureUnitButton:SetAttribute("*statebutton-P", "X");
    casterSecureUnitButton:SetAttribute("*statebutton-X", xString);
    casterSecureUnitButton:SetAttribute("*statebutton-HC", hcString);
    casterSecureUnitButton:SetAttribute("*statebutton-H", hString);
    casterSecureUnitButton:SetAttribute("*statebutton-M", mString);
    casterSecureUnitButton:SetAttribute("*type*", "click");
    casterSecureUnitButton:SetAttribute("*clickbutton*", selectedCasterSecureUnitButton);
    casterSecureUnitButton:SetAttribute("*statebutton-C", cString);
    for j=1,actionListSize do
      casterSecureUnitButton:SetAttribute("*clickbutton-C"..j, selectedCasterSecureUnitButtonBis);
      casterSecureUnitButton:SetAttribute("*clickbutton-M"..j, selectedCasterSecureUnitButton);
      casterSecureUnitButton:SetAttribute("*clickbutton-HC"..j, selectedSecureUnitButton);
    end

    local casterSecureUnitButtonBis = CreateFrame("Button", nil, parent, "SecureUnitButtonTemplate");
    casterSecureUnitButtonBisList[i] = casterSecureUnitButtonBis;
    casterSecureStateHeader:SetAttribute("addchild", casterSecureUnitButtonBis);    
    casterSecureUnitButtonBis:SetAttribute("*type*", "click");
    --  debugSecureUnitButton("SB"..i, casterSecureUnitButtonBis);

    for j=1,actionListSize do    
      for k=1,actionListSize do
        casterSecureUnitButtonBis:SetAttribute("*newstate-"..j.."M"..k, j..":"..k..";"..k..":"..j);
      end
    end    
    
    -- debugSecureUnitButton("Sn", casterSecureUnitButton);

    -- keep 4 state (open & not stuck => 0, open & stuck => 1, close & not stuck => 2, close & stuck => 3 )
    local secureStateHeader = CreateFrame("Frame", nil, UIParent, "SecureStateHeaderTemplate");
    secureStateHeaderList[i] = secureStateHeader;
    secureStateHeader:SetAllPoints();

    secureStateHeader.actionBar = actionBar;
    secureStateHeader.position = i;
    function secureStateHeader:StateChanged(e)
      e = tonumber(e);
      self.actionBar.state.visibilityList[self.position] = e;
    end
  
    local secureUnitButton = actionBarRenderer:CreateSecureUnitButton(secureStateHeader, i);
    secureUnitButtonList[i] = secureUnitButton;
    secureUnitButton:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    
    casterSecureStateHeader.secureUnitButton = secureUnitButton;
    casterSecureStateHeader.actionBar = actionBar;
    casterSecureStateHeader.position = i;
    function casterSecureStateHeader:StateChanged(e)
      e = tonumber(e);
      actionBarRenderer:ActionChanged(self.secureUnitButton, self.actionBar.actionList[e]);
      self.actionBar.state.actionList[self.position] = e;
    end

    casterSecureStateHeader:SetAttribute("state", tostring(actionBarState.actionList[i]));
    secureStateHeader:SetAttribute("state", tostring(actionBarState.visibilityList[i]));
    
    secureUnitButton:SetAttribute("type", "click");
    -- first * is mandatory to transmit modifier (shift)
    secureUnitButton:SetAttribute("*statebutton1", "0-1:HC;3:C");
    secureUnitButton:SetAttribute("ctrl-statebutton1", "P");
    secureUnitButton:SetAttribute("ctrl-shift-statebutton1", "X");
    secureUnitButton:SetAttribute("*statebutton2", "0-1:H;3:M");
    secureUnitButton:SetAttribute("*clickbutton*", casterSecureUnitButton);
    secureUnitButton:SetAttribute("hidestates", "2,4");
    secureStateHeader:SetAttribute("addchild", secureUnitButton);
    
    
    local secureUnitButtonBis = CreateFrame("Button", nil, parent, "SecureUnitButtonTemplate");
    secureUnitButtonBisList[i] = secureUnitButtonBis;
    secureStateHeader:SetAttribute("addchild", secureUnitButtonBis);
    secureUnitButtonBis:SetAttribute("type", "click");
    for j=1,actionListSize do
        secureUnitButtonBis:SetAttribute("*newstate-HC"..j,"0:2;1:3");
    end
    secureUnitButtonBis:SetAttribute("*newstate-H","0:2;1:3");
    secureUnitButtonBis:SetAttribute("*newstate-S","2:0;3:1");
    secureUnitButtonBis:SetAttribute("*statebutton-A", "2:D;3:A");
    secureUnitButtonBis:SetAttribute("*newstate-D","*:3");
    secureUnitButtonBis:SetAttribute("*statebutton-R", "2:R;3:E");
    secureUnitButtonBis:SetAttribute("*newstate-E","*:2");
  end
  
  local firstSecureUnitButtonBis = secureUnitButtonBisList[1];
  selectedSecureUnitButton:SetAttribute("*clickbutton*", firstSecureUnitButtonBis);
  -- debugSecureUnitButton("TBmax", firstSecureUnitButtonBis);


  -- finish link between buttons
  for i=1,actionListSize-1 do
    casterSecureUnitButtonList[i+1]:SetAttribute("*clickbutton-X", casterSecureUnitButtonList[i]);

    secureUnitButtonBisList[i]:SetAttribute("*clickbutton*", secureUnitButtonBisList[i+1]);
    secureUnitButtonBisList[i]:SetAttribute("*clickbutton-D", "");
    secureUnitButtonBisList[i]:SetAttribute("*clickbutton-E", "");
    secureUnitButtonBisList[i+1]:SetAttribute("*clickbutton-R", secureUnitButtonBisList[i]);
    casterSecureUnitButtonBisList[i+1]:SetAttribute("*clickbutton*", casterSecureUnitButtonBisList[i]);
    for j=1,actionListSize do  
      casterSecureUnitButtonList[i]:SetAttribute("*clickbutton-X"..j, casterSecureUnitButtonBisList[i+1]);
      casterSecureUnitButtonBisList[i]:SetAttribute("*newstate-"..j.."X", "*:"..j);
      casterSecureUnitButtonBisList[i]:SetAttribute("*clickbutton-"..j.."X", "");
      local x2String = "";
      for k=1,actionListSize do
        casterSecureUnitButtonBisList[i]:SetAttribute("*statebutton-"..k.."X"..j, "*:"..j.."X");
        casterSecureUnitButtonBisList[i+1]:SetAttribute("*newstate-"..k.."X"..j, "*:"..k);
        x2String = x2String..k..":"..j.."X"..k..";";
      end  
      casterSecureUnitButtonBisList[i+1]:SetAttribute("*statebutton-X"..j, x2String);
    end
  end


  local lastSecureUnitButtonBis = secureUnitButtonBisList[actionListSize];
  selectedSecureUnitButton:SetAttribute("*clickbutton-R", lastSecureUnitButtonBis);    
  lastSecureUnitButtonBis:SetAttribute("*clickbutton-S", selectedCasterSecureUnitButtonBis);
    
  for i=1,actionListSize do
    lastSecureUnitButtonBis:SetAttribute("*statebutton-HC"..i, "W"..i);
    lastSecureUnitButtonBis:SetAttribute("*newstate-W"..i,"0:2;1:3");
    lastSecureUnitButtonBis:SetAttribute("*clickbutton-W"..i, selectedCasterSecureUnitButtonBis);
  end
  -- debugSecureUnitButton("TBmin", lastSecureUnitButtonBis);
   
  local lastCasterSecureUnitButtonBis = casterSecureUnitButtonBisList[actionListSize];
  for i=1,actionListSize do
    for j=1,actionListSize do
      selectedCasterSecureUnitButton:SetAttribute("*clickbutton-"..i.."M"..j, lastCasterSecureUnitButtonBis);
    end
  end

  -- keep it to disable/enable action  
  actionBar.casterSecureStateHeaderList = casterSecureStateHeaderList;
  actionBar.secureStateHeaderList = secureStateHeaderList;
  actionBar.selectedSecureStateHeader = selectedSecureStateHeader;
  actionBar.selectedCasterSecureStateHeader = selectedCasterSecureStateHeader;
  
  for i=1,actionListSize do
    local action = actionList[i];
    -- can be directly enable, that why we register at the end
    action:RegisterEvent("ACTION_AVAILABILITY_CHANGED", actionBar, i);
  end
  
  return actionBar;  
end

