--[[
    EnvironmentLib
    Copyright (C) 2008 Gael Lalire
    Contact: gael.lalire@laposte.net

    EnvironmentLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    EnvironmentLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with EnvironmentLib.  If not, see <http://www.gnu.org/licenses/>.
--]]

local environmentLib = LibStub:NewLibrary("EnvironmentLib-1.0", 1);
if not environmentLib then
  return;
end

local factoryLib = LibStub("FactoryLib-1.0");
local eventHandlerLib = LibStub("EventHandlerLib-1.0");

-- Environment extends EventHandler
local Environment, environment = factoryLib.Factory.CreateSingletonType(eventHandlerLib.EventHandlerFactory);
environmentLib.Environment = Environment;
environmentLib.environment = environment;

-- unique instance : environment
environment.unrankedSpell = {};
environment.bookSize = 0;
environment.knownSpell = {};
-- managedEvent
environment.managedEvent = {["SPELLS_LEARNT"] = {["need"] = {"SPELLS_CHANGED", "PLAYER_ENTERING_WORLD"}},
                            ["SPELLS_FORGOTTEN"] = {["need"] = {"SPELLS_CHANGED"}},
                            ["UPDATE"] = {["need"] = {}},
                            ["UNIT_INVENTORY_CHANGED"] = {["need"] = {"UNIT_INVENTORY_CHANGED", "PLAYER_ENTERING_WORLD"}},
                           };
-- hookEvent
environment.hookEvent = {};

for _,v in pairs(environment.managedEvent) do
  v.used = false;
  for i=1,#v.need do
    local hookEventName = v.need[i];
    local hookEvent = environment.hookEvent[hookEventName];
    if hookEvent == nil then
      hookEvent = {};
      hookEvent.used = false;
      hookEvent.managedEventCount = 0;
      environment.hookEvent[hookEventName] = hookEvent;
    end
  end
end

local hookFrame = CreateFrame("Frame", nil, UIParent);
hookFrame.environment = environment;

hookFrame:SetScript("OnEvent", function (frame, eventName, ...)
  if eventName=="SPELLS_CHANGED" then
    frame.environment:SPELLS_CHANGED();
  elseif eventName=="PLAYER_ENTERING_WORLD" then
    frame.environment:SPELLS_CHANGED(); -- manage /reloadui
    frame.environment:Fire("UNIT_INVENTORY_CHANGED", "player"); -- we simulate
    frame.environment:Fire("PLAYER_ENTERING_WORLD");
  else
    -- This is not managed event (simply do each call)
    frame.environment:Fire(eventName, ...);
  end
end);

hookFrame:SetScript("OnUpdate", function (frame, ...)
  frame.environment:Fire("UPDATE");
end);

function Environment:OnUsed(eventName)
  local managedEvent = self.managedEvent[eventName];
  if managedEvent~=nil then
    managedEvent.used = true;
    for i=1,#managedEvent.need do
      local hookEventName = managedEvent.need[i];
      local hookEvent = self.hookEvent[hookEventName];
      if hookEvent.managedEventCount == 0 and hookEvent.used == false then
        hookFrame:RegisterEvent(hookEventName);
      end
      hookEvent.managedEventCount = hookEvent.managedEventCount + 1;
    end
  else
    local hookEvent = self.hookEvent[eventName];
    if hookEvent~=nil then
      hookEvent.used = true;
      if hookEvent.managedEventCount == 0 then
        hookFrame:RegisterEvent(eventName);
      end
    else
      -- not managed or hooked event
      hookFrame:RegisterEvent(eventName);      
    end  
  end
end

function Environment:OnUnused(eventName)
  local managedEvent = self.managedEvent[eventName];
  if managedEvent~=nil then
    managedEvent.used = false;
    for i=1,#managedEvent.need do
      local hookEventName = managedEvent.need[i];
      local hookEvent = self.hookEvent[hookEventName];
      hookEvent.managedEventCount = hookEvent.managedEventCount - 1;
      if hookEvent.managedEventCount == 0 and hookEvent.used == false then
        hookFrame:UnregisterEvent(hookEventName);
      end      
    end
  else
    local hookEvent = self.hookEvent[eventName];
    if hookEvent~=nil then
      hookEvent.used = false;
      if hookEvent.managedEventCount == 0 then
        hookFrame:UnregisterEvent(eventName);
      end
    else
      -- not managed or hooked event
      hookFrame:UnregisterEvent(eventName);      
    end  
  end
end

function Environment:UnitFullName(unit)
  local name, roy = UnitName(unit);
  if name~=nil then
    if roy~=nil then
      name = name.."-"..roy;    
    end
  end
  return name;
end


function Environment:FindPath(targetName)
  if UnitName("player") == targetName then
    return "player";
  end
  if self:UnitFullName("focus") == targetName then
    return "focus";
  end
  for i=1,4 do
    if self:UnitFullName("party"..i) == targetName then
      return "party"..i;
    end
  end
  for i=1,40 do
    if self:UnitFullName("raid"..i) == targetName then
      return "raid"..i;
    end
  end
  if self:UnitFullName("target") == targetName then
    return "target";
  end
  if self:UnitFullName("targettarget") == targetName then
    return "targettarget";
  end
  return nil;
end


function Environment:FillBook()
  local z = 1;
  local n, r = GetSpellName(z, BOOKTYPE_SPELL);
  while n do
    if self.knownSpell[n] then
      -- get the rank (if no rank then rank=1)
      local rank = environment:RankOfString(r);
      if rank == -1 then
        self.unrankedSpell[n] = {};
        rank = 1;
      end
      self.knownSpell[n][rank] = z;
    end
    z = z + 1;
    n, r = GetSpellName(z, BOOKTYPE_SPELL);
  end
end

-- cancel buffs
function environment:CancelPlayerBuffs(...)
  local buffs = {...};
  local doCancel = {};
  for i=1, #buffs do
    CancelPlayerBuff(buffs[i]);
  end
end

function Environment:UnitBuff(unit, buffName)
  local z, res = 1, {""};
  while res[1] do
    res = {UnitBuff(unit, z)};
    if res[1] == buffName then 
      return unpack(res);
    else
      z = z+1;
    end
  end
  return nil;
end


function Environment:GetSpellRank(spellName)
  local res = self.knownSpell[spellName];
  if not res then
    res = {};
    self.knownSpell[spellName] = res;
    self:FillBook();
  end
  return res;
end


function Environment:IsUnrankedSpell(spellName)
  self:GetSpellRank(spellName); -- fill book
  return self.unrankedSpell[spellName] ~= nil;
end

function Environment:RankOfString(stringRank)
  local s, e = string.find(stringRank, "(%d+)");
  if s==nil then
    return -1;
  else
    return tonumber(string.sub(stringRank, s, e));
  end
end

-- do not keep this value (may changed after SPELLS_FORGOTTEN)
-- this value is the only way to display spell tooltip
function Environment:GetSpellId(spellName, rank)
  if rank==nil then
    rank = self:GetSpellMaxRank(spellName);
  end
  -- a spell with no rank will be backup like a spell with one rank
  if rank == -1 then
    rank = 1;  
  end
  return self:GetSpellRank(spellName)[rank];
end

-- this value may evolve after SPELLS_LEARNT
function Environment:GetSpellMaxRank(spellName)
  local size = #self:GetSpellRank(spellName);
  if size == 1 then
    if self.unrankedSpell[spellName] ~= nil then
      return -1;
    end 
  end
  return size;
end

function Environment:GetTalentRank(talentName, talentBranch)
  local nt = GetNumTalents(talentBranch);
  for i=1, nt do
    local n,_,_,_, r, m = GetTalentInfo(talentBranch, i);
    if n == talentName then
      return r,m; 
    end
  end
  return 0, -1;
end

function Environment:SPELLS_CHANGED()
  local newBookSize = 0; 
  for i = 1, MAX_SKILLLINE_TABS do
    local name, _, offset, numSpells = GetSpellTabInfo(i);   
    if not name then
      break;
    end
    newBookSize = newBookSize + numSpells;
  end
      
  if newBookSize > self.bookSize then
    self.bookSize = newBookSize;          
    -- fill book
    self:FillBook()
    self:Fire("SPELLS_LEARNT");        
  elseif newBookSize < self.bookSize then
    self.bookSize = newBookSize;
    -- we delete book (because we forgot some spells)
    for n,_ in pairs(self.knownSpell) do
      self.knownSpell[n] = {};
    end
    -- reconstruct book
    self:FillBook()
    self:Fire("SPELLS_FORGOTTEN");    
  end
end

hookFrame:Show();
